import 'package:airbridge_flutter_sdk/airbridge.dart';
import 'package:airbridge_flutter_sdk_example/basic/part.dart';
import 'package:airbridge_flutter_sdk_example/basic/setting_row.dart' as Basic;
import 'package:airbridge_flutter_sdk_example/storage.dart';
import 'package:auto_size_text/auto_size_text.dart';

import 'package:flutter/material.dart';

class Setting extends StatefulWidget {
    @override
    State<StatefulWidget> createState() {
        return _State();
    }
}

class _State extends State<Setting> {
    Function sessionTimeoutCallback;
    Function deeplinkFetchTimeoutCallback;
    Function isUserInfoHashedCallback;
    Function isTrackingEnabledCallback;

    final sessionTimeoutController = TextEditingController();
    final deeplinkFetchTimeoutController = TextEditingController();
    bool isUserInfoHashed = true;
    bool isTrackingEnabled = true;

    _State() {
        sessionTimeoutCallback = (sessionTimeout) {
            Airbridge.setting.setSessionTimeout(
                sessionTimeout
            );
            setState(() {
                sessionTimeoutController.text
                = sessionTimeout.toString();
            });
        };
        deeplinkFetchTimeoutCallback = (deeplinkFetchTimeout) {
            Airbridge.setting.setDeeplinkFetchTimeout(
                deeplinkFetchTimeout
            );
            setState(() {
                deeplinkFetchTimeoutController.text
                = deeplinkFetchTimeout.toString();
            });
        };
        isUserInfoHashedCallback = (isUserInfoHashed) {
            Airbridge.setting.setIsUserInfoHashed(isUserInfoHashed);
            setState(() {
                this.isUserInfoHashed = isUserInfoHashed;
            });
        };
        isTrackingEnabledCallback = (isTrackingEnabled) {
            Airbridge.setting.setIsTrackingEnabled(isTrackingEnabled);
            setState(() {
                this.isTrackingEnabled = isTrackingEnabled;
            });
        };
    }

    @override
    void initState() {
        super.initState();

        Storage.addOnSet(
            'sessionTimeout',
            sessionTimeoutCallback
        );
        Storage.addOnSet(
            'deeplinkFetchTimeout',
            deeplinkFetchTimeoutCallback
        );
        Storage.addOnSet(
            'isTrackingEnabled',
            isTrackingEnabledCallback
        );
        Storage.addOnSet(
            'isUserInfoHashed',
            isUserInfoHashedCallback
        );

        setState(() {
            sessionTimeoutController.text
            = Storage.get('sessionTimeout').toString();
            deeplinkFetchTimeoutController.text
            = Storage.get('deeplinkFetchTimeout').toString();
            isUserInfoHashed
            = Storage.get('isUserInfoHashed');
            isTrackingEnabled
            = Storage.get('isTrackingEnabled');
        });
    }

    @override
    void dispose() {
        super.dispose();

        Storage.removeOnSet(
            'sessionTimeout',
            sessionTimeoutCallback
        );
        Storage.removeOnSet(
            'deeplinkFetchTimeout',
            deeplinkFetchTimeoutCallback
        );
        Storage.removeOnSet(
            'isUserInfoHashed',
            isUserInfoHashedCallback
        );
        Storage.removeOnSet(
            'isTrackingEnabled',
            isTrackingEnabledCallback
        );
    }

    @override
    Widget build(BuildContext context) {
        return _Container(
            onTap: () {
                FocusScope.of(context).unfocus();
            },
            child: Column(
                children: [
                    _MillisecondSettingRow(
                        name: 'Set timeout of session',
                        controller: sessionTimeoutController,
                        onPressed: () {
                            int timeout = int.parse(
                                sessionTimeoutController.text
                            );
                            if (timeout == null) {
                                return;
                            }

                            Airbridge.setting.setSessionTimeout(timeout);

                            FocusScope.of(context).unfocus();
                        },
                    ),
                    _MillisecondSettingRow(
                        name: 'Set timeout of deeplink-fetch',
                        controller: deeplinkFetchTimeoutController,
                        onPressed: () {
                            int timeout = int.parse(
                                deeplinkFetchTimeoutController.text
                            );
                            if (timeout == null) {
                                return;
                            }

                            Airbridge.setting.setDeeplinkFetchTimeout(timeout);

                            FocusScope.of(context).unfocus();
                        },
                    ),
                    _BoolSettingRow(
                        name: 'Hash user-information ?',
                        status: isUserInfoHashed ? 'YES' : 'NO',
                        onPressed: () {
                            Storage.set(
                                'isUserInfoHashed',
                                !isUserInfoHashed
                            );
                        },
                    ),
                    _BoolSettingRow(
                        name: 'Enable tracking ?',
                        status: isTrackingEnabled ? 'YES' : 'NO',
                        onPressed: () {
                            Storage.set(
                                'isTrackingEnabled',
                                !isTrackingEnabled
                            );
                        },
                    ),
                    _BoolSettingRow(
                        name: 'Reset',
                        status: 'Reset',
                        onPressed: () {
                            Storage.set('sessionTimeout', 15000);
                            Storage.set('deeplinkFetchTimeout', 3000);
                            Storage.set('isTrackingEnabled', true);
                            Storage.set('isUserInfoHashed', true);
                        },
                    ),
                ],
            ),
        );
    }

    void show(String title, String content) {
        Future.delayed(
            Duration.zero,
                () {
                showDialog(
                    context: context,
                    builder: (context) {
                        return AlertDialog(
                            title: Text(title),
                            content: Text(content),
                        );
                    },
                );
            }
        );
    }
}

class _AutoSizeText extends StatelessWidget {

    final String text;

    _AutoSizeText(this.text);

    @override
    Widget build(BuildContext context) {
        return AutoSizeText(
            text,
            maxLines: 1,
            minFontSize: 8,
        );
    }
}

class _Container extends StatelessWidget {

    final Widget child;
    final Function onTap;

    _Container({this.child, this.onTap});

    @override
    Widget build(BuildContext context) {
        return Container(
            padding: EdgeInsets.all(6.0),
            child: SingleChildScrollView(
                child: GestureDetector(
                    onTap: onTap,
                    child: child,
                ),
            )
        );
    }
}

class _MillisecondSettingRow extends StatelessWidget {

    final String name;
    final GlobalKey<FormState> key;
    final TextEditingController controller;
    final Function onPressed;

    _MillisecondSettingRow({
        this.name,
        this.key,
        this.controller,
        this.onPressed
    });

    @override
    Widget build(BuildContext context) {
        return Part(
            name,
            child: Basic.SettingRow(
                child: Container(
                    alignment: Alignment.centerLeft,
                    padding: EdgeInsets.all(6.0),
                    child: Form(
                        child: TextFormField(
                            controller: controller,
                            keyboardType: TextInputType.number,
                            decoration: InputDecoration(
                                hintText: 'Enter milliseconds'
                            ),
                            validator: (text) {
                                if (int.parse(text) == null) {
                                    return 'not number';
                                }

                                return null;
                            },
                        )
                    ),
                ),
                status: 'Set',
                backColor: Colors.blueGrey,
                textColor: Colors.white,
                onPressed: onPressed,
            ),
        );
    }
}

class _BoolSettingRow extends StatelessWidget {

    final String name;
    final String status;
    final Function onPressed;

    _BoolSettingRow({
        this.name,
        this.status,
        this.onPressed
    });

    @override
    Widget build(BuildContext context) {
        return Card(
            child: Container(
                padding: EdgeInsets.all(6.0),
                child: Basic.SettingRow(
                    child: Container(
                        alignment: Alignment.centerLeft,
                        padding: EdgeInsets.all(6.0),
                        child: _AutoSizeText(
                            name
                        ),
                    ),
                    status: status,
                    backColor: Colors.blueGrey,
                    textColor: Colors.white,
                    onPressed: onPressed,
                ),
            ),
        );
    }
}